package hikapro.com.backpack;

import android.app.Activity;
import android.app.Fragment;
import android.app.FragmentManager;
import android.app.FragmentTransaction;
import android.os.Bundle;
import android.util.Log;

import hikapro.com.backpack.model.DetailModel;
import hikapro.com.backpack.model.ItemModel;
import hikapro.com.backpack.model.SetModel;
import hikapro.com.backpack.model.entities.Item;
import hikapro.com.backpack.model.entities.Set;
import hikapro.com.backpack.presenter.ItemDetailPresenter;
import hikapro.com.backpack.presenter.ItemListPresenter;
import hikapro.com.backpack.presenter.SetListPresenter;
import hikapro.com.backpack.view.View;
import hikapro.com.backpack.view.fragments.ItemDetailFragment;
import hikapro.com.backpack.view.fragments.ItemListFragment;
import hikapro.com.backpack.view.fragments.SetListFragment;

public class MainActivity extends Activity implements View.ActivityCallback {

    private FragmentManager fragmentManager;


    private final StateMaintainer stateMaintainer =
            new StateMaintainer(getFragmentManager(), MainActivity.class.getName());

    // life cycle -->

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        fragmentManager = getFragmentManager();

        stateMaintainer.init();


        if (fragmentManager.getBackStackEntryCount() == 0 && savedInstanceState == null) {
            startSetListFragment();
        } else {
            Fragment fragment = fragmentManager.findFragmentByTag(SetListFragment.class.getName());
            if (fragment != null) {
                SetListFragment     view        = (SetListFragment) fragment;
                SetListPresenter    presenter   = stateMaintainer.get(SetListPresenter.class.getName());
                SetModel            model       = stateMaintainer.get(SetModel.class.getName());

                view.setPresenter(presenter);
                presenter.setView(view);
                presenter.setModel(model);
                model.setPresenter(presenter);
            }

            fragment = fragmentManager.findFragmentByTag(ItemListFragment.class.getName());
            if (fragment != null) {
                ItemListFragment     view        = (ItemListFragment) fragment;
                ItemListPresenter    presenter   = stateMaintainer.get(ItemListPresenter.class.getName());
                ItemModel            model       = stateMaintainer.get(ItemModel.class.getName());

                view.setPresenter(presenter);
                presenter.setView(view);
                presenter.setModel(model);
                model.setPresenter(presenter);

            }

            fragment = fragmentManager.findFragmentByTag(ItemDetailFragment.class.getName());
            if (fragment != null) {
                ItemDetailFragment  view        = (ItemDetailFragment) fragment;
                ItemDetailPresenter presenter   = stateMaintainer.get(ItemDetailPresenter.class.getName());
                DetailModel         model       = stateMaintainer.get(DetailModel.class.getName());

                view.setPresenter(presenter);
                presenter.setView(view);
                presenter.setModel(model);
                model.setPresenter(presenter);
            }
        }
        Log.i("On create", "Activity");
    }

    @Override
    protected void onStart() {
        super.onStart();
        Log.i("onStart", "Activity");
    }

    @Override
    protected void onRestart() {
        super.onRestart();
        Log.i("onRestart", "Activity");
    }

    @Override
    protected void onResume() {
        super.onResume();
        Log.i("onResume", "Activity");
    }

    @Override
    protected void onPause() {
        super.onPause();
        Log.i("onPause", "Activity");
    }

    @Override
    protected void onStop() {
        super.onStop();
        Log.i("onStop", "Activity");
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        Log.i("onDestroy", "Activity");
    }
    // life cycle <--


    @Override
    public void startSetListFragment() {

        SetListFragment     view        = SetListFragment.construct();
        SetListPresenter    presenter   = new SetListPresenter();
        SetModel            model       = new SetModel();

        view.setPresenter(presenter);
        //presenter.setView(view);
        presenter.setModel(model);
        model.setPresenter(presenter);

        replaceFragment(view, false, SetListFragment.class.getName(), false, false);
        stateMaintainer.put(presenter);
        stateMaintainer.put(model);

    }

    @Override
    public void startItemListFragment(Set set) {

        ItemListFragment    view        = ItemListFragment.newFromSet(set);
        ItemListPresenter   presenter   = new ItemListPresenter();
        ItemModel           model       = new ItemModel();

        view.setPresenter(presenter);
        presenter.setView(view);
        presenter.setModel(model);
        model.setPresenter(presenter);

        replaceFragment(view, true, ItemListFragment.class.getName(), true, true);

        stateMaintainer.put(presenter);
        stateMaintainer.put(model);
    }

    @Override
    public void startItemDetailFragment(Item item) {

        ItemDetailFragment  view        = ItemDetailFragment.newFromItem(item);
        ItemDetailPresenter presenter   = new ItemDetailPresenter();
        DetailModel         model       = new DetailModel();

        view.setPresenter(presenter);
        presenter.setView(view);
        presenter.setModel(model);
        model.setPresenter(presenter);

        replaceFragment(view, true, ItemDetailFragment.class.getName(), true, true);

        stateMaintainer.put(presenter);
        stateMaintainer.put(model);
    }

    private void replaceFragment(Fragment fragment, boolean addBackStack, String tag,
                                 boolean transition, boolean x) {
        FragmentTransaction transaction = fragmentManager.beginTransaction();
        if (transition) {
            if (x)
                transaction.setCustomAnimations(R.animator.slide_in_left, R.animator.slide_out_right,
                        R.animator.slide_in_right, R.animator.slide_out_left);
        }

        transaction.replace(R.id.container, fragment, tag);
        if (addBackStack)
            transaction.addToBackStack(null);
        transaction.setTransition(FragmentTransaction.TRANSIT_FRAGMENT_FADE);
        transaction.commit();
    }
}
